@@ -12,8 +12,10 @@ from unittest.runner import TextTestResult as _TextTestResult
 
 import portage
 from portage import os
+from portage.util import no_color
 from portage import _encodings
 from portage import _unicode_decode
+from portage.output import colorize
 from portage.proxy.objectproxy import ObjectProxy
 
 
@@ -68,7 +70,7 @@ def main():
 
     argv0 = Path(sys.argv[0])
 
-    usage = "usage: %s [options] [tests to run]" % argv0.name
+    usage = f"usage: {argv0.name} [options] [tests to run]"
     parser = argparse.ArgumentParser(usage=usage)
     parser.add_argument(
         "-l", "--list", help="list all tests", action="store_true", dest="list_tests"
@@ -77,7 +79,7 @@ def main():
     options = parser.parse_args(args=sys.argv)
 
     if (
-        os.environ.get("NOCOLOR") in ("yes", "true")
+        no_color(os.environ)
         or os.environ.get("TERM") == "dumb"
         or not sys.stdout.isatty()
     ):
@@ -88,7 +90,7 @@ def main():
         for mydir in getTestDirs(basedir):
             testsubdir = mydir.name
             for name in getTestNames(mydir):
-                print("%s/%s/%s.py" % (testdir, testsubdir, name))
+                print(f"{testdir}/{testsubdir}/{name}.py")
         return os.EX_OK
 
     if len(options.tests) > 1:
@@ -119,7 +121,7 @@ def getTestFromCommandLine(args, base_path):
         f = realpath.relative_to(path)
 
         if not f.name.startswith("test") or not f.suffix == ".py":
-            raise Exception("Invalid argument: '%s'" % arg)
+            raise Exception(f"Invalid argument: '{arg}'")
 
         mymodule = f.stem
         result.extend(getTestsFromFiles(path, base_path, [mymodule]))
@@ -180,23 +182,47 @@ class TextTestResult(_TextTestResult):
     """
 
     def __init__(self, stream, descriptions, verbosity):
-        super(TextTestResult, self).__init__(stream, descriptions, verbosity)
+        super().__init__(stream, descriptions, verbosity)
         self.todoed = []
         self.portage_skipped = []
 
+    def addSuccess(self, test):
+        super(_TextTestResult, self).addSuccess(test)
+        if self.showAll:
+            self.stream.writeln(colorize("GOOD", "ok"))
+        elif self.dots:
+            self.stream.write(colorize("GOOD", "."))
+            self.stream.flush()
+
+    def addError(self, test, err):
+        super(_TextTestResult, self).addError(test, err)
+        if self.showAll:
+            self.stream.writeln(colorize("BAD", "ERROR"))
+        elif self.dots:
+            self.stream.write(colorize("HILITE", "E"))
+            self.stream.flush()
+
+    def addFailure(self, test, err):
+        super(_TextTestResult, self).addFailure(test, err)
+        if self.showAll:
+            self.stream.writeln(colorize("BAD", "FAIL"))
+        elif self.dots:
+            self.stream.write(colorize("BAD", "F"))
+            self.stream.flush()
+
     def addTodo(self, test, info):
         self.todoed.append((test, info))
         if self.showAll:
-            self.stream.writeln("TODO")
+            self.stream.writeln(colorize("BRACKET", "TODO"))
         elif self.dots:
-            self.stream.write(".")
+            self.stream.write(colorize("BRACKET", "."))
 
     def addPortageSkip(self, test, info):
         self.portage_skipped.append((test, info))
         if self.showAll:
-            self.stream.writeln("SKIP")
+            self.stream.writeln(colorize("WARN", "SKIP"))
         elif self.dots:
-            self.stream.write(".")
+            self.stream.write(colorize("WARN", "."))
 
     def printErrors(self):
         if self.dots or self.showAll:
@@ -248,17 +274,17 @@ class TestCase(unittest.TestCase):
                 testMethod()
                 ok = True
             except unittest.SkipTest as e:
-                result.addPortageSkip(self, "%s: SKIP: %s" % (testMethod, str(e)))
+                result.addPortageSkip(self, f"{testMethod}: SKIP: {str(e)}")
             except self.failureException:
                 if self.portage_skip is not None:
                     if self.portage_skip is True:
-                        result.addPortageSkip(self, "%s: SKIP" % testMethod)
+                        result.addPortageSkip(self, f"{testMethod}: SKIP")
                     else:
                         result.addPortageSkip(
-                            self, "%s: SKIP: %s" % (testMethod, self.portage_skip)
+                            self, f"{testMethod}: SKIP: {self.portage_skip}"
                         )
                 elif self.todo:
-                    result.addTodo(self, "%s: TODO" % testMethod)
+                    result.addTodo(self, f"{testMethod}: TODO")
                 else:
                     result.addFailure(self, sys.exc_info())
             except (KeyboardInterrupt, SystemExit):
@@ -297,13 +323,13 @@ class TestCase(unittest.TestCase):
                 excName = excClass.__name__
             else:
                 excName = str(excClass)
-            raise self.failureException("%s not raised: %s" % (excName, msg))
+            raise self.failureException(f"{excName} not raised: {msg}")
 
     def assertNotExists(self, path):
         """Make sure |path| does not exist"""
         path = Path(path)
         if path.exists():
-            raise self.failureException("path exists when it should not: %s" % path)
+            raise self.failureException(f"path exists when it should not: {path}")
 
 
 class TextTestRunner(unittest.TextTestRunner):
@@ -331,7 +357,7 @@ class TextTestRunner(unittest.TextTestRunner):
         )
         self.stream.writeln()
         if not result.wasSuccessful():
-            self.stream.write("FAILED (")
+            self.stream.write(colorize("BAD", "FAILED") + " (")
             failed = len(result.failures)
             errored = len(result.errors)
             if failed:
@@ -342,7 +368,7 @@ class TextTestRunner(unittest.TextTestRunner):
                 self.stream.write("errors=%d" % errored)
             self.stream.writeln(")")
         else:
-            self.stream.writeln("OK")
+            self.stream.writeln(colorize("GOOD", "OK"))
         return result
 
 
